@@ -4,10 +4,12 @@
 
 import logging
 
-from yaml import full_load as load_yaml
+from copy import deepcopy
+
+from yaml import full_load as load_yaml  # type: ignore
 from pkgutil import get_data
 from os import listdir, path
-from pyaml import dump as dump_yaml
+from pyaml import dump as dump_yaml  # type: ignore
 
 
 from .other_files import OtherFiles
@@ -20,9 +22,9 @@ from .util import (
     deref_multi,
     find_tei_directory,
     strip_strings_in_dict,
+    extract_unique_name,
 )
 
-
 log = logging.getLogger(__name__)
 
 
@@ -135,6 +137,10 @@ class YAMLConfigBase(object):
             )
             return default
 
+    def exists(self):
+        log.debug("This check for existance is very rough!!!")
+        return bool(self.content)
+
 
 class MainConfig(YAMLConfigBase):
     def __init__(self, *args, **kw):
@@ -151,12 +157,29 @@ class MainConfig(YAMLConfigBase):
                 )
         return request
 
-    def exists(self):
-        log.debug("This check for existance is very rough!!!")
-        return bool(self.content)
+    def _check_files(self):
+        save = False
+        for subproject in self.content["subprojects"]:
+            if "files" not in subproject:
+                subproject["files"] = sorted(get_files(subproject["inpath"]))
+                save = True
+        if save:
+            self.save()
 
     def get_subprojects(self):
-        return self.content["subprojects"]
+        if self.exists():
+            self._check_files()
+            return deepcopy(self.content["subprojects"])
+        else:
+            return []
+
+    def get_subproject(self, name=None, inpath=None):
+        for sp in self.get_subprojects():
+            if name and sp["name"] == name:
+                return sp
+            if inpath and sp["inpath"] == inpath:
+                return sp
+        return None
 
     @property
     def min_hitrate(self):
@@ -176,12 +199,19 @@ class MainConfig(YAMLConfigBase):
             file.write(dump_yaml(self.content))
 
     def update(
-        self, inputpath=None, sourcepath=None, tei_directory=None, **kw
+        self,
+        inputpath=None,
+        sourcepath=None,
+        tei_directory=None,
+        tei_directories=None,
+        **kw,
     ):
         if sourcepath and tei_directory:
             tei_directories = find_tei_directory(sourcepath, tei_directory, 0)
         elif inputpath:
             tei_directories = inputpath.split(",")
+        elif tei_directories is not None:
+            pass
         else:
             log.error("Neither 'sourcepath' nor 'inputpath' is defined!")
             raise ValueError
@@ -189,7 +219,11 @@ class MainConfig(YAMLConfigBase):
         log.debug("tei_directories: %s" % tei_directories)
         existing_subprojects = []
         for sp in self.get_subprojects():
-            existing_subprojects.append(sp["name"])
+            if sp["inpath"] in tei_directories:
+                existing_subprojects.append(sp["name"])
+            else:
+                # shutil.rmtree(sp["basepath"])
+                self.content["subprojects"].remove(sp)
 
         # ToDo: merge the following code with the same code in
         # MainConfigTemplate.render
@@ -235,6 +269,7 @@ class MainConfig(YAMLConfigBase):
                     "name": input_name,
                     "outpath": outpath,
                     "basepath": basepath,
+                    "files": sorted(get_files(prepared_path)),
                 }
             )
 
@@ -242,15 +277,6 @@ class MainConfig(YAMLConfigBase):
         self.save()
 
 
-class SynergyConfig(YAMLConfigBase):
-    def __init__(self, *args, **kw):
-        super().__init__("synergy.yaml", *args, **kw)
-
-    @property
-    def filepath(self):
-        return self.content["filepath"]
-
-
 class CollectionConfig(YAMLConfigBase, CollectionMixin):
     def __init__(self, *args, **kw):
         super().__init__("collection.yaml", *args, **kw)
@@ -319,93 +345,25 @@ class YAMLConfigTemplate(YAMLConfigBase):
         with open(self.path, "w") as file:
             file.write(dump_yaml(self.content, sort_keys=False))
 
-    def process_proposals(
-        self, attribs, compare_to="sources", orig_content=None
-    ):
-        main_config = MainConfig(self.projectpath)
-
-        log.debug("Compare to: %s" % compare_to)
-        for attrib in attribs:
-            orig_attrib = None
-            if orig_content:
-                orig_attrib = orig_content.get(attrib)
-            if attribs[attrib]:
-                attribs[attrib]["value"] = None
-                if "proposals" in attribs[attrib]:
-                    # we pop out the 'proposals' and we want to remove them
-                    # from the config eiterway
-                    proposals = attribs[attrib].pop("proposals")
-                    if not proposals:
-                        continue
-
-                    # check if the attribute is already defined in the existing
-                    # config AND set this
-                    # orig_content is None when 'overwrite' is requested
-                    if orig_attrib:
-                        attribs[attrib] = orig_attrib
-                        continue
-
-                    # init some statistics
-                    min_hitrate = main_config.min_hitrate  # in %
-                    winner = None
-                    for _xpath_prop in proposals:
-                        xpath_prop = split_xpath(_xpath_prop)
-                        log.debug("[process_proposals] %s" % xpath_prop)
-                        if compare_to == "sources":
-                            hitrate = self.synergy_analyzer.hits_in_sources(
-                                xpath_prop["xpath"]
-                            )
-                            if hitrate > min_hitrate:
-                                winner = xpath_prop["full_path"]
-                                min_hitrate = hitrate
-                        elif compare_to == "synergy":
-                            # pass
-                            if (
-                                self.synergy_analyzer.parser.find(
-                                    xpath_prop["xpath"]
-                                )
-                                is not None
-                            ):
-                                winner = xpath_prop["full_path"]
-                    if winner is not None and "xpath" in attribs[attrib]:
-                        log.debug(
-                            "[process_proposals] And the winner is `%s` having \
-%s percent hitrate"
-                            % (winner, min_hitrate)
-                        )
-                        attribs[attrib]["xpath"] = winner
-                else:
-                    # ok...there are no proposals at this level
-                    # but maybe at the next level, so let's do some
-                    # recursion
-                    self.process_proposals(
-                        attribs[attrib],
-                        compare_to=compare_to,
-                        orig_content=orig_attrib,
-                    )
-
 
 class MainConfigTemplate(YAMLConfigTemplate):
     def __init__(self, *args, **kw):
         super().__init__("main.yaml", *args, **kw)
 
     def render(
-        self, inputpath=None, sourcepath=None, tei_directory=None, **kw
+        self,
+        inputpath=None,
+        tei_directories=None,
+        sourcepath=None,
+        tei_directory=None,
+        **kw,
     ):
-        # self.content["projectname"] = kw.get("projectpath")
-        # print(kw.get("projectpath"))
-        # breakpoint()
-        # if not self.content["projectname"]:
-        #     # ToDo: remove usage of 'projectname' completely!
-        #     log.warning(
-        #         "MainConfigTemplate.render - Deprecated usage of projectname"
-        #     )
-        #     self.content["projectname"] = kw.get("projectname")
-
         if sourcepath and tei_directory:
             tei_directories = find_tei_directory(sourcepath, tei_directory, 0)
         elif inputpath:
             tei_directories = inputpath.split(",")
+        elif tei_directories:
+            pass
         else:
             log.error("Neither 'sourcepath' nor 'inputpath' is defined!")
             raise ValueError
@@ -415,22 +373,33 @@ class MainConfigTemplate(YAMLConfigTemplate):
         # ToDo: merge the following code with the same code in
         # MainConfig.update
         subprojects = []
+        unique_names = extract_unique_name(tei_directories)
         for p in tei_directories:
             prepared_path = prepare_path(p)
             splitted_pp = prepared_path.split("/")
             length = 3 if len(splitted_pp) >= 3 else len(splitted_pp)
 
-            if sourcepath:
-                # try to find name of subproject only if there is more
-                # than one subproject
-                if len(tei_directories) > 1:
-                    input_name = find_name_of_subproject(
-                        kw["projectname"], p, tei_directory
-                    )
-                if len(tei_directories) <= 1 or not input_name:
-                    input_name = sourcepath.split("/")[-1]
-            else:
-                input_name = "_".join(splitted_pp[-length:])
+            # Try to get a unique name for the subproject
+            # 1. check if the name is in the list of unique names
+            input_name = None
+            for unique_name in unique_names:
+                if unique_name in p:
+                    input_name = unique_name
+                    break
+            # 2. if not, try to find a name in the path
+            if not input_name:
+                # 2.1 if sourcepath is given try to find name of subproject
+                # only if there is more than one subproject
+                if sourcepath:
+                    if len(tei_directories) > 1:
+                        input_name = find_name_of_subproject(
+                            kw["projectname"], p, tei_directory
+                        )
+                    if len(tei_directories) <= 1 or not input_name:
+                        input_name = sourcepath.split("/")[-1]
+                # 2.2 if sourcepath is not given, use the last parts of path
+                else:
+                    input_name = "_".join(splitted_pp[-length:])
 
             outpath = prepare_path(
                 (
@@ -454,6 +423,7 @@ class MainConfigTemplate(YAMLConfigTemplate):
                     "name": input_name,
                     "outpath": outpath,
                     "basepath": basepath,
+                    "files": sorted(get_files(prepared_path)),
                 }
             )
 
@@ -466,31 +436,10 @@ class MainConfigTemplate(YAMLConfigTemplate):
         return MainConfig(projectpath=self.projectpath)
 
 
-class SynergyConfigTemplate(YAMLConfigTemplate):
-    def __init__(self, synergy_analyzer, *args, **kw):
-        self.synergy_analyzer = synergy_analyzer
-        super().__init__("synergy.yaml", *args, **kw)
-
-    def render(self):
-        ca_parser = self.synergy_analyzer.parser
-        ca_parser.set_config(self)
-
-        self.process_proposals(
-            self.content,
-            compare_to="synergy",
-        )
-
-        self.content["filepath"] = self.synergy_analyzer.write_tree(
-            "%s/synergy.xml" % self.subproject["basepath"]
-        )
-        self.save()
-        log.info("%s initialized!" % self.path)
-
-
 class CollectionConfigTemplate(YAMLConfigTemplate, CollectionMixin):
-    def __init__(self, synergy_analyzer, projectname=None, *args, **kw):
-        self.synergy_analyzer = synergy_analyzer
+    def __init__(self, projectname=None, files=[], *args, **kw):
         self.projectname = projectname
+        self.files = files
         super().__init__(
             "collection.yaml",
             *args,
@@ -505,26 +454,58 @@ class CollectionConfigTemplate(YAMLConfigTemplate, CollectionMixin):
         else:
             getattr(self, key)(value=default_value)
 
+    def hits_in_files(self, xpath):
+        # all_sources = [self.base] + self.targets
+        count = 0
+        for tree in self.files:
+            if tree.find(xpath) is not None:
+                count += 1
+        return round((count * 100) / len(self.files), 2)
+
+    def process_proposals(self, attribs, orig_content=None):
+        main_config = MainConfig(self.projectpath)
+
+        for attrib in attribs:
+            orig_attrib = None
+            if orig_content:
+                orig_attrib = orig_content.get(attrib)
+            if attribs[attrib]:
+                attribs[attrib]["value"] = None
+                if "proposals" in attribs[attrib]:
+                    # we pop out the 'proposals' and we want to remove them
+                    # from the config eiterway
+                    proposals = attribs[attrib].pop("proposals")
+                    if not proposals:
+                        continue
+
+                    # init some statistics
+                    min_hitrate = main_config.min_hitrate  # in %
+                    winner = None
+                    for _xpath_prop in proposals:
+                        xpath_prop = split_xpath(_xpath_prop)
+                        log.debug("[process_proposals] %s" % xpath_prop)
+                        hitrate = self.hits_in_files(xpath_prop["xpath"])
+                        if hitrate > min_hitrate:
+                            winner = xpath_prop["full_path"]
+                            min_hitrate = hitrate
+                    if winner is not None and "xpath" in attribs[attrib]:
+                        log.debug(
+                            "[process_proposals] And the winner is `%s` having \
+%s percent hitrate"
+                            % (winner, min_hitrate)
+                        )
+                        attribs[attrib]["xpath"] = winner
+                else:
+                    # ok...there are no proposals at this level
+                    # but maybe at the next level, so let's do some
+                    # recursion
+                    self.process_proposals(
+                        attribs[attrib],
+                        orig_content=orig_attrib,
+                    )
+
     def render(self, overwrite=False):
         collection_config = CollectionConfig(self.subproject["basepath"])
-        syan_parser = self.synergy_analyzer.parser
-
-        for key in [
-            "short_title",
-            "long_title",
-        ]:
-            new_value = getattr(syan_parser, key)
-            old_value = (
-                getattr(collection_config, key)()
-                if collection_config.content
-                else None
-            )
-            _func = getattr(self, key)
-            _func(
-                new_value
-                if overwrite or old_value in [None, ""]
-                else old_value
-            )
 
         if not self.short_title():
             self.short_title(self.subproject["name"])
@@ -537,8 +518,10 @@ class CollectionConfigTemplate(YAMLConfigTemplate, CollectionMixin):
             else:
                 self.long_title(self.short_title())
 
-        for t in self.synergy_analyzer.sourcefiles:
-            self.elements.append({"fullpath": t.uri, "filename": t.file_name})
+        for target in self.subproject["files"]:
+            self.elements.append(
+                {"fullpath": target, "filename": target.split("/")[-1]}
+            )
 
         # running through proposals and set xpath if hitrate is above
         for section in ["attributes", "eltec_specs"]:
